﻿using System;
using System.Windows;

// Reference File: InfragisticsSL4.v10.3
using Infragistics;
// Reference File: InfragisticsSL4.Persistence.v10.3
using Infragistics.Persistence;
using Infragistics.Persistence.Primitives;


using System.IO;
using System.IO.IsolatedStorage;

namespace MyPersistanceTest.Views
{
    public class DmcSettingsToStorage
    {
        #region constructor
        public DmcSettingsToStorage()
        {
            //_settings.SavePersistenceOptions = PersistenceOption.OnlySpecified;

            //checks that the dorner max has been requested for this application
            IncreaseStorage(DmcIsoQuotaBytes);
            CreateIsoFldr(IsoFldr);
        }
        #endregion  //constructor

        #region properties
        //public const long DmcIsoQuotaBytes = 52428800;  //50MB 51200KB
        public const long DmcIsoQuotaBytes = 20971520;  //20MB 20480KB

        private const string IsoFldr = @"DmcConfigSettings\";

        // Determines which properties are saved
        public enum SaveProperties
        {
            Grid,
            WidthOnly
        }

        #endregion  //properties

        #region public methods
        public void Save(DependencyObject obj, string objname, SaveProperties saveprops)
        //public void Save(Object obj, string objname, SaveProperties saveprops)
        {
            MemoryStream memoryStream = new MemoryStream();
            // set the properites to save
            //SetSaveProperties(saveprops);

            //_memoryStream = PersistenceManager.Save(obj, _settings);
            //memoryStream = PersistenceManager.Save(obj as DependencyObject);
            memoryStream = PersistenceManager.Save(obj);

            string isofile = IsoFldr + objname;

            // start working with the Isolated Storage
            using (IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForApplication())
            {
                // if the file exists, delete it before measuring for available space
                if (iso.FileExists(isofile)) iso.DeleteFile(isofile);

                //see if there is enough free space for the file
                if (memoryStream.Length >= iso.AvailableFreeSpace)
                {
                    //ask for more
                    IncreaseStorage(memoryStream.Length + DmcIsoQuotaBytes);
                }

                //write to the file
                using (IsolatedStorageFileStream isos = new IsolatedStorageFileStream(isofile, FileMode.OpenOrCreate, iso))
                {
                    isos.Write(memoryStream.ToArray(), 0, (int)memoryStream.Length);
                }
            }

        }

        public void Load(DependencyObject obj, string objname)
        //public void Load(Object obj, string objname)
        {
            string isofile = IsoFldr + objname;

            // start working with the Isolated Storage
            using (IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForApplication())
            {
                // if the file exists, load the objects settings
                if (iso.FileExists(isofile))
                {
                    //read the file
                    using (IsolatedStorageFileStream isos = new IsolatedStorageFileStream(isofile, FileMode.Open, iso))
                    {
                        if (isos.Length > 0)
                        {
                            isos.Position = 0;
                            //PersistenceManager.Load(obj as DependencyObject, isos);
                            PersistenceManager.Load(obj, isos);
                        }
                    }

                }

            }
        }
        #endregion  //public methods

        #region commands

        #endregion  //commands

        #region event handlers
        #endregion  //event handlers

        #region events
        #endregion  //events

        #region private variables
        private PersistenceSettings _settings = new PersistenceSettings();
        #endregion  //private variables

        #region private methods
        private void IncreaseStorage(long spacerequest)
        {
            //Tries to increase an Isolated Storage quota and raises an exception if the user declines the request

            using (IsolatedStorageFile isf = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (isf.Quota < spacerequest)
                {
                    if (!isf.IncreaseQuotaTo(spacerequest))
                    {
                        throw new IsolatedStorageException();
                    }
                }
            }
        }

        private void CreateIsoFldr(string fldr)
        {
            // If the folder does not exist, it will be created
            using (IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (!iso.DirectoryExists(fldr))
                {
                    iso.CreateDirectory(fldr);
                }
            }
        }

        private void SetSaveProperties(SaveProperties saveprops)
        {
            _settings.PropertySettings.Clear();

            switch (saveprops)
            {
                case SaveProperties.Grid:
                    _settings.PropertySettings.Add(CreatePropertyNamePersistenceInfo("Columns"));
                    break;
                case SaveProperties.WidthOnly:
                    _settings.PropertySettings.Add(CreatePropertyNamePersistenceInfo("Width"));
                    break;
                default:
                    break;
            }
        }

        private PropertyNamePersistenceInfo CreatePropertyNamePersistenceInfo(String propertyName)
        {
            PropertyNamePersistenceInfo pnpi = new PropertyNamePersistenceInfo();
            pnpi.PropertyName = propertyName;

            return pnpi;
        }

        #endregion  //private methods
    }
}
